-- Properties (subjects of Agda -> FOL translation)
-- Authors: Andreas Abel, Ulf Norell
-- Created: 2004-11-16

-- atoms

(==) (A : Set) : A -> A -> Prop

-- connectives

data True : Prop where
  tt : True

data False : Prop where

(==>) : Prop -> Prop -> Prop
Not : Prop -> Prop
(/\) : Prop -> Prop -> Prop
(\/) : Prop -> Prop -> Prop
Exists (A : Set) : (A -> Prop) -> Prop

(<==) : Prop -> Prop -> Prop
p <== q = q ==> p

(/=) (A : Set) : A -> A -> Prop
x /= y = Not (x == y)

(<=>) : Prop -> Prop -> Prop
P <=> Q = (P ==> Q) /\ (Q ==> P)

ForAll (A : Set) : (A -> Prop) -> Prop
ForAll {A} B = (x : A) -> B x

infixl 5 ==, /=
infixl 4 /\
infixl 3 \/
infixr 2 ==>, <==
infix 1 <=>

-- axioms

-- Proof rules

impI (A, B : Prop) : (A -> B) -> (A ==> B)
impE (A, B : Prop) : (A ==> B) -> (A -> B)

existsE (A : Set)(C : Type) (P : A -> Prop) : Exists P -> ((w : A) -> P w -> C) -> C
existsI (A : Set) (P : A -> Prop) : (w : A) -> P w -> Exists P

notI (A : Prop) : (A -> False) -> Not A
notE (A : Prop) : Not A -> (A -> False)

absE (A : Set) : False -> A

andI  (P, Q : Prop) : P -> Q -> P /\ Q
andE1 (P, Q : Prop) : P /\ Q -> P
andE2 (P, Q : Prop) : P /\ Q -> Q

extensionality (A, B : Set) :
    (f, g : A -> B) -> ((x : A) -> f x == g x) ==> f == g

